@@ -17,7 +17,7 @@ To use::
 
 import math
 import numpy as np
-from typing import Any
+from typing import Any, Type
 from spatialmath import base
 from spatialmath.pose3d import SO3, SE3
 from spatialmath.smuserlist import SMUserList
@@ -983,9 +983,9 @@ class UnitQuaternion(Quaternion):
 
             elif isinstance(s, np.ndarray) and s.shape[1] == 4:
                 if norm:
-                    self.data = [base.qnorm(x) for x in s]
+                    self.data = [base.unit(x) for x in s]
                 else:
-                    self.data = [x for x in s]
+                    self.data = [base.qpositive(x) for x in s]
 
             elif isinstance(s, SO3):
                 # UnitQuaternion(x) x is SO3 or SE3
@@ -1211,7 +1211,7 @@ class UnitQuaternion(Quaternion):
         return cls([base.rand() for i in range(0, N)], check=False)
 
     @classmethod
-    def Eul(cls, angles, *, unit='rad'):
+    def Eul(cls, *angles, unit='rad'):
         r"""
         Construct a new unit quaternion from Euler angles
 
@@ -1222,9 +1222,13 @@ class UnitQuaternion(Quaternion):
         :return: unit-quaternion
         :rtype: UnitQuaternion instance
 
-        ``UnitQuaternion.Eul(𝚪)`` is a unit quaternion that describes the 3D
-        rotation defined by a 3-vector of Euler angles :math:`\Gamma = (\phi, \theta, \psi)` which correspond to consecutive rotations about the Z, Y, Z axes
-        respectively.
+        - ``UnitQuaternion.Eul(𝚪)`` is a unit quaternion that describes the 3D
+          rotation defined by a 3-vector of Euler angles :math:`\Gamma = (\phi,
+          \theta, \psi)` which correspond to consecutive rotations about the Z,
+          Y, Z axes respectively.
+
+        - ``UnitQuaternion.Eul(φ, θ, ψ)`` as above but the angles are provided
+          as three scalars.
 
         Example:
 
@@ -1235,10 +1239,13 @@ class UnitQuaternion(Quaternion):
 
         :seealso: :func:`~spatialmath.quaternion.UnitQuaternion.RPY`, :func:`~spatialmath.pose3d.SE3.eul`, :func:`~spatialmath.pose3d.SE3.Eul`, :func:`~spatialmath.base.transforms3d.eul2r`
         """
+        if len(angles) == 1:
+            angles = angles[0]
+
         return cls(base.r2q(base.eul2r(angles, unit=unit)), check=False)
 
     @classmethod
-    def RPY(cls, angles, *, order='zyx', unit='rad'):
+    def RPY(cls, *angles, order='zyx', unit='rad'):
         """
         Construct a new unit quaternion from roll-pitch-yaw angles
 
@@ -1251,18 +1258,27 @@ class UnitQuaternion(Quaternion):
         :return: unit-quaternion
         :rtype: UnitQuaternion instance
 
-        ``UnitQuaternion.RPY(𝚪)`` is a unit quaternion that describes the 3D rotation defined by a  3-vector of roll, pitch, yaw angles :math:`\Gamma = (r, p, y)`
-        which correspond to successive rotations about the axes specified by ``order``:
+        - ``UnitQuaternion.RPY(𝚪)`` is a unit quaternion that describes the 3D
+          rotation defined by a  3-vector of roll, pitch, yaw angles
+          :math:`\Gamma = (r, p, y)` which correspond to successive rotations
+          about the axes specified by ``order``:
 
-            - ``'zyx'`` [default], rotate by yaw about the z-axis, then by pitch about the new y-axis,
-              then by roll about the new x-axis.  Convention for a mobile robot with x-axis forward
-              and y-axis sideways.
-            - ``'xyz'``, rotate by yaw about the x-axis, then by pitch about the new y-axis,
-              then by roll about the new z-axis. Convention for a robot gripper with z-axis forward
-              and y-axis between the gripper fingers.
-            - ``'yxz'``, rotate by yaw about the y-axis, then by pitch about the new x-axis,
-              then by roll about the new z-axis. Convention for a camera with z-axis parallel
-              to the optic axis and x-axis parallel to the pixel rows.
+            - ``'zyx'`` [default], rotate by yaw about the z-axis, then by pitch
+              about the new y-axis, then by roll about the new x-axis.
+              Convention for a mobile robot with x-axis forward and y-axis
+              sideways.
+            - ``'xyz'``, rotate by yaw about the x-axis, then by pitch about the
+              new y-axis, then by roll about the new z-axis. Convention for a
+              robot gripper with z-axis forward and y-axis between the gripper
+              fingers.
+            - ``'yxz'``, rotate by yaw about the y-axis, then by pitch about the
+              new x-axis, then by roll about the new z-axis. Convention for a
+              camera with z-axis parallel to the optic axis and x-axis parallel
+              to the pixel rows.
+
+
+        - ``UnitQuaternion.RPY(⍺, β, 𝛾)`` as above but the angles are provided
+          as three scalars.
 
         Example:
 
@@ -1273,6 +1289,9 @@ class UnitQuaternion(Quaternion):
 
         :seealso: :func:`~spatialmath.quaternion.UnitQuaternion.Eul`, :func:`~spatialmath.pose3d.SE3.rpy`, :func:`~spatialmath.pose3d.SE3.RPY`, :func:`~spatialmath.base.transforms3d.rpy2r`
         """
+        if len(angles) == 1:
+            angles = angles[0]
+
         return cls(base.r2q(base.rpy2r(angles, unit=unit, order=order)), check=False)
 
     @classmethod
@@ -1724,23 +1743,42 @@ class UnitQuaternion(Quaternion):
         """
         return left.binop(right, lambda x, y: not base.isequal(x, y, unitq=True), list1=False)
 
-    def interp(self, s=0, dest=None, shortest=False):
+    def __matmul__(left, right):  # lgtm[py/not-named-self] pylint: disable=no-self-argument
+        """
+        Overloaded @ operator
+
+        :return: product :rtype: UnitQuaternion
+
+        - ``q1 @ q2`` is the Hamilton product of ``q1`` and ``q2``, both unit
+          quaternions, followed by explicit normalization.
+
+        - `` q1 @= q2`` as above.
+
+        .. note:: This operator is functionally equivalent to ``*`` but is more
+            costly.  It is useful for cases where a pose is incrementally update
+            over many cycles.
+        """
+        return left.__class__(left.binop(right, lambda x, y: base.unit(base.qqmul(x, y))))
+
+    def interp(self, end, s=0, shortest=False):
         """
         Interpolate between two unit quaternions
 
-        :param dest: destination unit quaternion
-        :type dest: UnitQuaternion
+        :param end: final unit quaternion
+        :type end: UnitQuaternion
         :param shortest: Take the shortest path along the great circle
-        :param s: interpolation in range [0,1]
-        :type s: float
+        :param s: interpolation coefficient, range 0 to 1, or number of steps
+        :type s: array_like or int
         :return: interpolated unit quaternion
         :rtype: UnitQuaternion instance
 
-        - ``q.interp(s)`` is a unit quaternion that is interpolated between
-          the identity quaternion and ``q``.  Spherical linear interpolation (slerp) is used.
+        - ``q0.interp(q1, s)`` is a unit quaternion that is interpolated between
+          ``q0`` when s=0 and ``q1`` when s=1. Spherical linear interpolation
+          (slerp) is used.  If ``s`` is an ndarray(n) then the result will be
+          a UnitQuaternion with n values.
 
-        - ``q1.interp(s, dest=q2)`` as above but interpolated between
-          ``q1`` and ``q2``.
+        - ``q0.interp(q1, N)`` interpolate between ``q0`` and ``q1`` in ``N``
+          steps.
 
         Example:
 
@@ -1750,35 +1788,31 @@ class UnitQuaternion(Quaternion):
             >>> q1 = UQ.Rx(0.3); q2 = UQ.Rz(-0.4)
             >>> print(q1)
             >>> print(q2)
-            >>> print(q1.interp(0, q2))    # this is q1
-            >>> print(q1.interp(1, q2))    # this is q2
-            >>> print(q1.interp(0.5, q2))  # this is in between
+            >>> q1.interp(q2, 0)    # this is q1
+            >>> q1.interp(q2, 1,)   # this is q2
+            >>> q1.interp(q2, 0.5)  # this is in between
+            >>> q = q1.interp(q2, 11)  # in 11 steps
+            >>> len(q)
+            >>> q[0]                # this is q1
+            >>> q[5]                # this is in between
+
+        .. note:: values of ``s`` are silently clipped to the range [0, 1]
 
         :seealso: :func:`~spatialmath.base.quaternions.slerp`
         """
-        # TODO vectorize
+        # TODO allow self to have len() > 1
 
-        if dest is not None:
-            # 2 quaternion form
-            assert isinstance(dest, UnitQuaternion)
-            if s == 0:
-                return self
-            elif s == 1:
-                return dest
-            q1 = self.vec
-            q2 = dest.vec
+        if isinstance(s, int) and s > 1:
+            s = np.linspace(0, 1, s)
         else:
-            # 1 quaternion form
-            if s == 0:
-                return UnitQuaternion()
-            elif s == 1:
-                return self
-
-            q1 = base.eye()
-            q2 = self.vec
-
-        assert 0 <= s <= 1, 's must be in interval [0,1]'
+            s = base.getvector(s)
+            s = np.clip(s, 0, 1)  # enforce valid values
 
+        # 2 quaternion form
+        if not isinstance(end, UnitQuaternion):
+            raise TypeError('end argument must be a UnitQuaternion')
+        q1 = self.vec
+        q2 = end.vec
         dot = base.inner(q1, q2)
 
         # If the dot product is negative, the quaternions
@@ -1789,16 +1823,120 @@ class UnitQuaternion(Quaternion):
                 q1 = - q1
                 dot = -dot
 
+        # shouldn't be needed by handle numerical errors: -eps, 1+eps cases
         dot = np.clip(dot, -1, 1)  # Clip within domain of acos()
-        theta_0 = math.acos(dot)  # theta_0 = angle between input vectors
-        theta = theta_0 * s  # theta = angle between v0 and result
-        if theta_0 == 0:
-            return UnitQuaternion(q1)
 
-        s1 = float(math.cos(theta) - dot * math.sin(theta) / math.sin(theta_0))
-        s2 = math.sin(theta) / math.sin(theta_0)
-        out = (q1 * s1) + (q2 * s2)
-        return UnitQuaternion(out)
+        theta_0 = math.acos(dot)  # theta_0 = angle between input vectors
+
+        qi = []
+        for sk in s:
+            theta = theta_0 * sk  # theta = angle between v0 and result
+
+            s1 = float(math.cos(theta) - dot * math.sin(theta) / math.sin(theta_0))
+            s2 = math.sin(theta) / math.sin(theta_0)
+            out = (q1 * s1) + (q2 * s2)
+            qi.append(out)
+
+        return UnitQuaternion(qi)
+
+    def interp1(self, s=0, shortest=False):
+        """
+        Interpolate a unit quaternions
+
+        :param shortest: Take the shortest path along the great circle
+        :param s: interpolation coefficient, range 0 to 1, or number of steps
+        :type s: array_like or int
+        :return: interpolated unit quaternion
+        :rtype: UnitQuaternion instance
+
+        - ``q.interp1(s)`` is a unit quaternion that is interpolated between
+          identity when s=0 and ``q`` when s=1. Spherical linear interpolation
+          (slerp) is used.  If ``s`` is an ndarray(n) then the result will be
+          a UnitQuaternion with n values.
+
+        - ``q.interp1(N)`` interpolate between identity and ``q1`` in ``N``
+          steps.
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import UnitQuaternion as UQ
+            >>> q = UQ.Rx(0.3)
+            >>> print(q)
+            >>> q.interp1(0)    # this is identity
+            >>> q.interp1(1)    # this is q
+            >>> q.interp1(0.5)  # this is in between
+            >>> qi = q.interp1(q2, 11)  # in 11 steps
+            >>> len(qi)
+            >>> qi[0]                # this is q1
+            >>> qi[5]                # this is in between
+
+        .. note:: values of ``s`` are silently clipped to the range [0, 1]
+
+        :seealso: :func:`~spatialmath.base.quaternions.slerp`
+        """
+        # TODO allow self to have len() > 1
+
+        if isinstance(s, int) and s > 1:
+            s = np.linspace(0, 1, s)
+        else:
+            s = base.getvector(s)
+            s = np.clip(s, 0, 1)  # enforce valid values
+
+        q = self.vec
+        dot = q[0]   # s
+
+        # If the dot product is negative, the quaternions
+        # have opposite handed-ness and slerp won't take
+        # the shorter path. Fix by reversing one quaternion.
+        if shortest:
+            if dot < 0:
+                q = - q
+                dot = -dot
+
+        # shouldn't be needed by handle numerical errors: -eps, 1+eps cases
+        dot = np.clip(dot, -1, 1)  # Clip within domain of acos()
+
+        theta_0 = math.acos(dot)  # theta_0 = angle between input vectors
+
+        qi = []
+        for sk in s:
+            theta = theta_0 * sk  # theta = angle between v0 and result
+
+            s1 = float(math.cos(theta) - dot * math.sin(theta) / math.sin(theta_0))
+            s2 = math.sin(theta) / math.sin(theta_0)
+            out = np.r_[s1, 0, 0, 0] + (q * s2)
+            qi.append(out)
+
+        return UnitQuaternion(qi)
+
+    def increment(self, w, normalize=False):
+        """
+        Quaternion incremental update
+
+        :param w: angular displacement, Euler vector
+        :type w: array_like(3)
+        :param normalize: normalize the result, defaults to False
+        :type normalize: bool, optional
+
+        .. note:: The object state is updated
+        """
+
+        # is (v, theta) or None
+        vt = base.unitvec_norm(w)
+
+        if vt is None:
+            # zero update
+            return
+    
+        ds = math.cos(vt[1] / 2)
+        dv = math.sin(vt[1] / 2) * vt[0]
+
+        updated = base.qqmul(self.A, np.r_[ds, dv])
+        if normalize:
+            updated = base.unit(updated)
+        self.data = [updated]
 
     def plot(self, *args, **kwargs):
         """
@@ -1841,6 +1979,10 @@ class UnitQuaternion(Quaternion):
 
         :see :func:`~spatialmath.base.transforms3d.tranimate`, :func:`~spatialmath.base.transforms3d.trplot`
         """
+        if len(self) > 1:
+            base.tranimate([base.q2r(q) for q in self.data], *args, **kwargs)
+        else:
+            base.tranimate(base.q2r(self._A), *args, **kwargs)
 
     def rpy(self, unit='rad', order='zyx'):
         """
@@ -1985,12 +2127,14 @@ class UnitQuaternion(Quaternion):
     #     """
     #     return Quaternion(s=0, v=math.acos(self.s) * base.unitvec(self.v))
 
-    def angle(self, other):
+    def angdist(self, other, metric=0):
         """
-        Angle between unit quaternions
+        Angular distance metric between unit quaternions
 
         :param other: second unit quaternion
         :type other: UnitQuaternion instance
+        :param metric: metric, default is 0
+        :type metric: int
         :raises TypeError: if other is not a UnitQuaternion
         :return: angle in radians
         :rtype: float
@@ -1998,21 +2142,48 @@ class UnitQuaternion(Quaternion):
         ``q1.angle(q2)`` is the geodesic norm, or geodesic distance between two
         unit quaternions and has units of angle.  We can consider it as the angle between two quaternions.
 
+        Several metrics are supported:
+
+        ======   ===============================================================
+        Metric   Details
+        ======   ===============================================================
+        0        :math:`1 - | \q_1 \sbullet \q_2 | \in [0, 1]`
+        1        :math:`\cos^{-1} | \q_1 \sbullet \q_2 | \in [0, \pi/2]`
+        2        :math:`2 \cos^{-1} | \q_1 \sbullet \q_2 | \in [0, \pi]`
+        3        :math:`2 \tan^{-1} \| \q_1 - \q_2\| / \|\q_1 + \q_2\| \in [0, \pi/2]`
+        4        :math:`\cos^{-1} \left( 2 (\q_1 \sbullet \q_2)^2 - 1\right) \in [0, 1]`
+        ======   ===============================================================
+
         Example:
 
         .. runblock:: pycon
 
             >>> from spatialmath import UnitQuaternion
             >>> q1 = UnitQuaternion.Rx(0.3)
-            >>> q2 = UnitQuaternion.Rx(0.3)
-            >>> print(q1.angle(q1))
-            >>> print(q1.angle(q2))
+            >>> q2 = UnitQuaternion.Ry(0.3)
+            >>> print(q1.angdist(q1))
+            >>> print(q1.angdist(q2))
 
+        .. note::
+            - metrics 2 and 3 are equivalent
+            - SMTB-MATLAB uses metric 3 for UnitQuaternion.angle()
+            - MATLAB's quaternion.dist() uses metric 2
         """
         if not isinstance(other, UnitQuaternion):
             raise TypeError('bad operand')
 
-        return math.acos(2 * self.inner(other) ** 2 - 1)
+        if metric == 0:
+            measure = lambda p, q: 1 - np.dot(p, q)
+        elif metric == 1:
+            measure =  lambda p, q: math.acos(np.dot(p, q))
+        elif metric == 2:
+            measure =  lambda p, q: 2 * math.acos(np.dot(p, q))
+        elif metric == 3:
+            measure =  lambda p, q: 2 * math.atan2(base.norm(p - q), base.norm(p + q))
+        elif metric == 4:
+            measure = lambda p, q: math.acos(2 * np.dot(p, q) ** 2 - 1)
+
+        return self.binop(other, measure)
 
     def SO3(self):
         """
@@ -2066,10 +2237,6 @@ if __name__ == '__main__':  # pragma: no cover
 
     import pathlib
 
-    q=UnitQuaternion.Rand()
-    a = q.log()
-    print(a, a.norm())
-    a = q.exp()
-    print(a, a.norm())
+    a = UnitQuaternion([0, 1, 0, 0])
 
     exec(open(pathlib.Path(__file__).parent.parent.absolute() / "tests" / "test_quaternion.py").read())  # pylint: disable=exec-used
