@@ -48,7 +48,7 @@ class TestUnitQuaternion(unittest.TestCase):
         qcompare(UnitQuaternion([0, 0, 0, 1]), np.r_[0, 0, 0, 1])
 
         qcompare(UnitQuaternion([2, 0, 0, 0]), np.r_[1, 0, 0, 0])
-        qcompare(UnitQuaternion([-2, 0, 0, 0]), np.r_[-1, 0, 0, 0])
+        qcompare(UnitQuaternion([-2, 0, 0, 0]), np.r_[1, 0, 0, 0])
 
         # from [S,V]
         qcompare(UnitQuaternion(1, [0, 0, 0]), np.r_[1, 0, 0, 0])
@@ -57,7 +57,7 @@ class TestUnitQuaternion(unittest.TestCase):
         qcompare(UnitQuaternion(0, [0, 0, 1]), np.r_[0, 0, 0, 1])
 
         qcompare(UnitQuaternion(2, [0, 0, 0]), np.r_[1, 0, 0, 0])
-        qcompare(UnitQuaternion(-2, [0, 0, 0]), np.r_[-1, 0, 0, 0])
+        qcompare(UnitQuaternion(-2, [0, 0, 0]), np.r_[1, 0, 0, 0])
 
         # from R
 
@@ -356,6 +356,16 @@ class TestUnitQuaternion(unittest.TestCase):
         # vector x scalar
         nt.assert_array_almost_equal(UnitQuaternion([ry, rz, rx]) * vy, np.c_[vy, -vx, vz])
 
+    def test_matmul(self):
+
+        rx = UnitQuaternion.Rx(pi / 2)
+        ry = UnitQuaternion.Ry(pi / 2)
+        rz = UnitQuaternion.Rz(pi / 2)
+
+        qcompare(rx @ ry, rx * ry)
+
+        qcompare(UnitQuaternion([ry, rz, rx]) @ UnitQuaternion([rx, ry, rz]), UnitQuaternion([ry * rx, rz * ry, rx * rz]))
+
     # def multiply_test_normalized(self):
 
     #     vx = [1, 0, 0]; vy = [0, 1, 0]; vz = [0, 0, 1]
@@ -414,8 +424,11 @@ class TestUnitQuaternion(unittest.TestCase):
 
         # , 3 angle
         qcompare(UnitQuaternion.RPY([0.1, 0.2, 0.3]).rpy(), [0.1, 0.2, 0.3])
+        qcompare(UnitQuaternion.RPY(0.1, 0.2, 0.3).rpy(), [0.1, 0.2, 0.3])
 
         qcompare(UnitQuaternion.Eul([0.1, 0.2, 0.3]).eul(), [0.1, 0.2, 0.3])
+        qcompare(UnitQuaternion.Eul(0.1, 0.2, 0.3).eul(), [0.1, 0.2, 0.3])
+
 
         qcompare(UnitQuaternion.RPY([10, 20, 30], unit='deg').R, rpy2r(10, 20, 30, unit='deg'))
 
@@ -492,19 +505,31 @@ class TestUnitQuaternion(unittest.TestCase):
         q = rx * ry * rz
 
         # from null
-        qcompare(q.interp(0), u)
-        qcompare(q.interp(1), q)
+        qcompare(q.interp1(0), u)
+        qcompare(q.interp1(1), q)
 
         #self.assertEqual(length(q.interp(linspace(0,1, 10))), 10)
         #self.assertTrue(all( q.interp([0, 1]) == [u, q]))
         # TODO vectorizing
 
-        q0_5 = q.interp(0.5)
+        q0_5 = q.interp1(0.5)
         qcompare(q0_5 * q0_5, q)
 
+        qq = rx.interp1(11)
+        self.assertEqual(len(qq), 11)
+
         # between two quaternions
-        qcompare(q.interp(0, dest=rx), q)
-        qcompare(q.interp(1, dest=rx), rx)
+        qcompare(q.interp(rx, 0), q)
+        qcompare(q.interp(rx, 1), rx)
+
+        # test vectorised results
+        qq = q.interp(rx, [0, 1])
+        self.assertEqual(len(qq), 2)
+        qcompare(qq[0], q)
+        qcompare(qq[1], rx)
+
+        qq = rx.interp(q, 11)
+        self.assertEqual(len(qq), 11)
 
         #self.assertTrue(all( q.interp([0, 1], dest=rx, ) == [q, rx]))
 
@@ -517,6 +542,25 @@ class TestUnitQuaternion(unittest.TestCase):
         # qcompare( qq(6), UnitQuaternion.Rx(pi) )
         # TODO interp
 
+    def test_increment(self):
+        q = UnitQuaternion()
+
+        q.increment([0, 0, 0])
+        qcompare(q, UnitQuaternion())
+
+        q.increment([0, 0, 0], normalize=True)
+        qcompare(q, UnitQuaternion())
+
+        for i in range(10):
+            q.increment([0.1, 0, 0])
+        qcompare(q, UnitQuaternion.Rx(1))
+
+        q = UnitQuaternion()
+        for i in range(10):
+            q.increment([0.1, 0, 0], normalize=True)
+        qcompare(q, UnitQuaternion.Rx(1))
+
+
     def test_eq(self):
         q1 = UnitQuaternion([0, 1, 0, 0])
         q2 = UnitQuaternion([0, -1, 0, 0])
